from abc import ABC, abstractmethod

from metripy.Metric.Code.FileMetrics import FileMetrics
from metripy.Tree.ModuleNode import ModuleNode


class AbstractLangAnalyzer(ABC):
    def __init__(self):
        self.files: list[str] = []
        self.modules: dict[str, ModuleNode] = {}

    @abstractmethod
    def set_files(self, files: list[str]) -> None:
        raise NotImplementedError

    @abstractmethod
    def is_needed(self) -> bool:
        pass

    def before_run(self) -> None:
        # build cache
        pass

    def after_run(self) -> None:
        # clear cache
        pass

    @abstractmethod
    def get_lang_name(self) -> str:
        raise NotImplementedError

    @abstractmethod
    def run(self) -> None:
        raise NotImplementedError

    @abstractmethod
    def get_metrics(self) -> list[FileMetrics]:
        metrics = []
        for module in self.modules.values():
            full_name = module.full_name

            if len(module.functions) > 0:
                totalCc = sum(function.complexity for function in module.functions)
                avgCcPerFunction = totalCc / len(module.functions)
                avgLocPerFunction = (
                    module.lloc - module.comments - len(module.functions)
                ) / len(module.functions)
            else:
                totalCc = 0
                avgCcPerFunction = 0
                avgLocPerFunction = 0
            maintainabilityIndex = module.maintainability_index

            file_metric = FileMetrics(
                full_name=full_name,
                loc=module.loc,
                totalCc=totalCc,
                avgCcPerFunction=avgCcPerFunction,
                maintainabilityIndex=maintainabilityIndex,
                avgLocPerFunction=avgLocPerFunction,
                class_nodes=module.classes,
                function_nodes=module.functions,
            )
            metrics.append(file_metric)

        return metrics



from collections import defaultdict

from tree_sitter_languages import get_parser


class TypescriptAstParser:
    def __init__(self):
        self.parser = get_parser("typescript")

    def _get_node_text(self, code: str, node) -> str:
        return code[node.start_byte : node.end_byte].decode("utf-8")

    def extract_structure(self, code: str) -> dict:
        tree = self.parser.parse(bytes(code, "utf8"))
        root_node = tree.root_node
        structure = defaultdict(list)
        structure["classes"] = {}
        structure["functions"] = []
        structure["enums"] = []

        def traverse(node, parent_class=None):
            if node.type == "class_declaration":
                class_name = None
                for child in node.children:
                    if child.type == "type_identifier":
                        class_name = self._get_node_text(code.encode(), child)
                        structure["classes"][class_name] = []
                for child in node.children:
                    traverse(child, class_name)

            elif node.type == "method_definition" and parent_class:
                for child in node.children:
                    if child.type == "property_identifier":
                        method_name = self._get_node_text(code.encode(), child)
                        structure["classes"][parent_class].append(method_name)

            elif node.type == "function_declaration":
                for child in node.children:
                    if child.type == "identifier":
                        function_name = self._get_node_text(code.encode(), child)
                        structure["functions"].append(function_name)

            elif node.type == "lexical_declaration":
                # Handle exported arrow functions like: export const foo = (...) => {...}
                for child in node.children:
                    if child.type == "variable_declarator":
                        identifier = None
                        for grandchild in child.children:
                            if grandchild.type == "identifier":
                                identifier = self._get_node_text(
                                    code.encode(), grandchild
                                )
                            elif grandchild.type == "arrow_function":
                                if identifier:
                                    structure["functions"].append(identifier)

            elif node.type == "enum_declaration":
                enum_name = None
                for child in node.children:
                    if child.type == "identifier":
                        enum_name = self._get_node_text(code.encode(), child)
                        structure["enums"].append(enum_name)

            for child in node.children:
                traverse(child, parent_class)

        traverse(root_node)
        return dict(structure)
